﻿Option Strict On
Imports Microsoft.Kinect

Class MainWindow
    Dim BgrPixel As Integer = CInt(PixelFormats.Bgr32.BitsPerPixel / 8) 'Bgr32形式は１ピクセルあたりのビット数が32bitのRGB形式。先頭から１バイト（8bit)ずつに青、緑、赤、アルファの情報が入っている

    Dim kinect As KinectSensor

  

    Private Sub MainWindow_Loaded(sender As Object, e As System.Windows.RoutedEventArgs) Handles Me.Loaded

        Try
            If KinectSensor.KinectSensors.Count = 0 Then
                MessageBox.Show("Kinectが接続されていません")
                Exit Sub
            Else
                kinect = KinectSensor.KinectSensors(0)
            End If

            kinect.ColorStream.Enable()
            kinect.DepthStream.Enable()
            kinect.SkeletonStream.Enable()

            AddHandler kinect.AllFramesReady, AddressOf kinect_AllFramesReady

            kinect.Start()
        Catch ex As Exception
            MessageBox.Show(ex.Message)
            Close()
        End Try

    End Sub




    Private Sub kinect_AllFramesReady(sender As Object, e As AllFramesReadyEventArgs)
        Try
            Dim kinect As KinectSensor = DirectCast(sender, KinectSensor)
            If kinect Is Nothing Then
                Return
            End If


            Using colorFrame As ColorImageFrame = e.OpenColorImageFrame()
                Using depthFrame As DepthImageFrame = e.OpenDepthImageFrame()
                    If (colorFrame IsNot Nothing) AndAlso (depthFrame IsNot Nothing) Then
                        Dim getBackgroundMask As Byte() = BackgroundMask(kinect, colorFrame, depthFrame)
                        Image1.Source = BitmapSource.Create(colorFrame.Width, colorFrame.Height, 96, 96, PixelFormats.Bgra32, Nothing, getBackgroundMask, colorFrame.Width * colorFrame.BytesPerPixel)

                    End If
                End Using
            End Using
        Catch ex As Exception
            MessageBox.Show(ex.Message)
        End Try
    End Sub


    Private Function BackgroundMask(kinect As KinectSensor, colorFrame As ColorImageFrame, depthFrame As DepthImageFrame) As Byte()
        Dim myColorStream As ColorImageStream = kinect.ColorStream
        Dim myDepthStream As DepthImageStream = kinect.DepthStream

        Dim colorPixel As Byte() = New Byte(colorFrame.PixelDataLength - 1) {}
        colorFrame.CopyPixelDataTo(colorPixel)

        'KINECT SDK 1.6 対応
        Dim depthPixel As DepthImagePixel() = New DepthImagePixel(depthFrame.PixelDataLength - 1) {}
        depthFrame.CopyDepthImagePixelDataTo(depthPixel)

        Dim colorPoint As ColorImagePoint() = New ColorImagePoint(depthFrame.PixelDataLength - 1) {}

        'KINECT SDK 1.6 対応
        kinect.CoordinateMapper.MapDepthFrameToColorFrame(myDepthStream.Format, depthPixel, myColorStream.Format, colorPoint)

        Dim opacityColor As Byte() = New Byte(colorPixel.Length - 1) {}

        For index As Integer = 0 To depthPixel.Length - 1
            'KINECT SDK 1.6対応
            Dim player As Integer = depthPixel(index).PlayerIndex


            Dim x As Integer = colorPoint(index).X
            Dim y As Integer = colorPoint(index).Y
            Dim colorIndex As Integer = ((y * depthFrame.Width) + x) * BgrPixel

            If player <> 0 Then
                opacityColor(colorIndex) = colorPixel(colorIndex) 'Blue
                opacityColor(colorIndex + 1) = colorPixel(colorIndex + 1) 'Green

                opacityColor(colorIndex + 2) = colorPixel(colorIndex + 2) 'Red
                opacityColor(colorIndex + 3) = &HFF 'Alpha
            End If
        Next

        Return opacityColor
    End Function

    Private Sub StopKinect(kinect As KinectSensor)
        If kinect Is Nothing = False Then
            If kinect.IsRunning = True Then
                RemoveHandler kinect.AllFramesReady, AddressOf kinect_AllFramesReady
                kinect.Stop()
                kinect.Dispose()
            End If
        End If
    End Sub


    Private Sub MainWindow_Closing(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles Me.Closing
        StopKinect(KinectSensor.KinectSensors(0))
    End Sub

End Class
